@@ -1,20 +1,18 @@
 package Net::IMAP::Client;
 
 use vars qw[$VERSION];
-$VERSION = '0.11';
+$VERSION = '0.2';
 
 use strict;
 use warnings;
 
 use List::Util qw( min max first );
-use List::MoreUtils qw( each_array );
 use IO::Socket::INET ();
 use IO::Socket::SSL ();
-use Socket qw( SO_KEEPALIVE );
 
 use Net::IMAP::Client::MsgSummary ();
 
-our $READ_BUFFER = 4096;
+our $SYSREAD_BUFFER = 4096;
 my %UID_COMMANDS = map { $_ => 1 } qw( COPY FETCH STORE SEARCH SORT THREAD );
 my %DEFAULT_ARGS = (
     uid_mode => 1,
@@ -38,18 +36,14 @@ sub new {
     bless $self, $class;
 
     $self->{notifications} = [];
-    eval {
-        $self->{greeting} = $self->_socket_getline;
-    };
-    return $@ ? undef : $self;
+    $self->{greeting} = $self->_socket_getline;
+    return $self;
 }
 
 sub DESTROY {
     my ($self) = @_;
-    eval {
-        $self->quit
-          if $self->_get_socket->opened;
-    };
+    $self->quit
+      if $self->_get_socket->opened;
 }
 
 sub uid_mode {
@@ -77,6 +71,7 @@ sub login {
 
 sub logout {
     my ($self) = @_;
+    $self->_send_cmd('EXPUNGE');
     $self->_send_cmd('LOGOUT');
     $self->_get_socket->close;
     return 1;
@@ -118,15 +113,6 @@ sub status {
         $_ = "STATUS $_ (MESSAGES RECENT UNSEEN UIDNEXT UIDVALIDITY)";
     }
     my $results = $self->_tell_imap2(@$a);
-
-    # remove "NO CLIENT BUG DETECTED" lines as they serve no
-    # purpose beyond the religious zeal of IMAP implementors
-    for my $row (@$results) {
-        if (@{$row->[1]} > 1) {
-            $row->[1] = [ grep { $_->[0] !~ /NO CLIENT BUG DETECTED: STATUS on selected mailbox:/ } @{$row->[1]} ];
-        }
-    }
-
     my %ret;
     my $name;
     foreach my $i (@$results) {
@@ -298,69 +284,44 @@ sub get_rfc822_body {
 
 sub get_part_body {
     my ($self, $msg, $part) = @_;
-    $part = "BODY[$part]";
-    my ($ok, $lines) = $self->_tell_imap(FETCH => "$msg $part", 1);
+    my ($ok, $lines) = $self->_tell_imap(FETCH => "$msg BODY[$part]", 1);
     if ($ok) {
-        # it can contain FLAGS notification, i.e. \Seen flag becomes set
-        my $tokens = _parse_tokens($lines->[0], 1);
-        my %hash = @{$tokens->[3]};
-        if ($hash{FLAGS}) {
-            $self->_handle_notification($tokens);
-        }
-        return $hash{$part};
+        return $lines->[0]->[1];
     }
     return undef;
 }
 
 sub get_parts_bodies {
     my ($self, $msg, $parts) = @_;
-    my $tmp = join(' ', map { "BODY[$_]" } @$parts);
-    my ($ok, $lines) = $self->_tell_imap(FETCH => "$msg ($tmp)", 1);
+    $parts = join(' ', map { "BODY[$_]" } @$parts);
+    my ($ok, $lines) = $self->_tell_imap(FETCH => "$msg ($parts)", 1);
     if ($ok) {
-        # it can contain FLAGS notification, i.e. \Seen flag becomes set
-        my $tokens = _parse_tokens($lines->[0], 1);
-        my %hash = @{$tokens->[3]};
-        if ($hash{FLAGS}) {
-            $self->_handle_notification($tokens);
+        my $a = $lines->[0];
+        my %ret = ();
+        for (my $i = 0; $i < @$a;) {
+            my ($foo, $bar) = ($a->[$i++], $a->[$i++]);
+            if ($foo =~ /BODY\[(.*?)\]\s*$/) {
+                $ret{$1} = $bar;
+            }
         }
-        my %ret = map {( $_, $hash{"BODY[$_]"} )} @$parts;
         return \%ret;
     }
     return undef;
 }
 
 sub get_summaries {
-    my ($self, $msg, $headers) = @_;
+    my ($self, $msg) = @_;
+    my @lines;
     if (!$msg) {
         $msg = '1:*';
     } elsif (ref $msg eq 'ARRAY') {
         $msg = join(',', @$msg);
     }
-    if ($headers) {
-        $headers = " BODY.PEEK[HEADER.FIELDS ($headers)]";
-    } else {
-        $headers = '';
-    }
-    my ($ok, $lp) = $self->_tell_imap(FETCH => qq[$msg (UID FLAGS INTERNALDATE RFC822.SIZE ENVELOPE BODYSTRUCTURE$headers)], 1);
+    my ($ok, $lp) = $self->_tell_imap(FETCH => qq[$msg (UID FLAGS INTERNALDATE RFC822.SIZE ENVELOPE BODYSTRUCTURE)], 1);
     if ($ok) {
         my @ret;
         foreach (@$lp) {
-            my $summary;
-            my $tokens = _parse_tokens($_); ## in form: [ '*', ID, 'FETCH', [ tokens ]]
-            if ($tokens->[2] eq 'FETCH') {
-                my %hash = @{$tokens->[3]};
-                if ($hash{ENVELOPE}) {
-                    # full fetch
-                    $summary = Net::IMAP::Client::MsgSummary->new(\%hash, undef, !!$headers);
-                    $summary->{seq_id} = $tokens->[1];
-                } else {
-                    # 'FETCH' (probably FLAGS) notification!
-                    $self->_handle_notification($tokens);
-                }
-            } else {
-                # notification!
-                $self->_handle_notification($tokens);
-            }
+            my $summary = $self->_make_summary(_parse_tokens($_));
             push @ret, $summary
               if $summary;
         }
@@ -370,184 +331,6 @@ sub get_summaries {
     }
 }
 
-sub fetch {
-    my ($self, $msg, $keys) = @_;
-    my $wants_many = undef;
-    if (ref $msg eq 'ARRAY') {
-        $msg = join(',', @$msg);
-        $wants_many = 1;
-    }
-    if (ref $keys eq 'ARRAY') {
-        $keys = join(' ', @$keys);
-    }
-    my ($ok, $lp) = $self->_tell_imap(FETCH => qq[$msg ($keys)], 1);
-    if ($ok) {
-        my @ret;
-        foreach (@$lp) {
-            my $tokens = _parse_tokens($_)->[3];
-            push @ret, { @$tokens };
-        }
-        return $wants_many || @ret > 1 ? \@ret : $ret[0];
-    }
-}
-
-sub create_folder {
-    my ($self, $folder) = @_;
-    my $quoted = $folder;
-    _string_quote($quoted);
-    my ($ok) = $self->_tell_imap(CREATE => $quoted);
-    return $ok;
-}
-
-# recursively removes any subfolders!
-sub delete_folder {
-    my ($self, $folder) = @_;
-    my $quoted = $folder . $self->separator . '*';
-    _string_quote($quoted);
-    my ($ok, $lines) = $self->_tell_imap(LIST => qq{"" $quoted});
-    if ($ok) {
-        my @subfolders;
-        foreach my $line (@$lines) {
-            my $tokens = _parse_tokens($line);
-            push @subfolders, $tokens->[4];
-        }
-        @subfolders = sort { length($b) - length($a) } @subfolders;
-        foreach (@subfolders) {
-            _string_quote($_);
-            ($ok) = $self->_tell_imap(DELETE => $_);
-        }
-        $quoted = $folder;
-        _string_quote($quoted);
-        ($ok) = $self->_tell_imap(DELETE => $quoted);
-    }
-    return $ok;
-}
-
-sub append {
-    my ($self, $folder, $rfc822, $flags, $date) = @_;
-    my $quoted = $folder;
-    _string_quote($quoted);
-    my $args = [ "$quoted " ];
-    if ($flags) {
-        # my @tmp = @$flags;
-        # $quoted = join(' ', map { _string_quote($_) } @tmp);
-        # push @$args, "($quoted) ";
-        push @$args, '(' . join(' ', @$flags) . ') ';
-    }
-    if ($date) {
-        my $tmp = $date;
-        _string_quote($tmp);
-        push @$args, "$tmp ";
-    }
-    push @$args, $rfc822;
-    my ($ok) = $self->_tell_imap(APPEND => $args, 1);
-    return $ok;
-}
-
-sub copy {
-    my ($self, $msg, $folder) = @_;
-    my $quoted = $folder;
-    _string_quote($quoted);
-    if (ref $msg eq 'ARRAY') {
-        $msg = join(',', @$msg);
-    }
-    my ($ok) = $self->_tell_imap(COPY => "$msg $quoted", 1);
-    return $ok;
-}
-
-sub get_flags {
-    my ($self, $msg) = @_;
-    my $wants_many = undef;
-    if (ref($msg) eq 'ARRAY') {
-        $msg = join(',', @$msg);
-        $wants_many = 1;
-    }
-    my ($ok, $lines) = $self->_tell_imap(FETCH => "$msg (UID FLAGS)", 1);
-    if ($ok) {
-        my %ret = map {
-            my $tokens = _parse_tokens($_)->[3];
-            my %hash = @$tokens;
-            $hash{UID} => $hash{FLAGS};
-        } @$lines;
-        return $wants_many ? \%ret : $ret{$msg};
-    }
-    return undef;
-}
-
-sub get_threads {
-    my ($self, $algo, $msg) = @_;
-    $algo ||= "REFERENCES";
-    my ($ok, $lines) = $self->_tell_imap(THREAD => "$algo UTF-8 ALL");
-    if ($ok) {
-        my $result = $lines->[0][0];
-        $result =~ s/^\*\s+THREAD\s+//;
-        my $parsed = _parse_tokens([ $result ]);
-        if ($msg) {
-            (my $left = $result) =~ s/\b$msg\b.*$//;
-            my $thr = 0;
-            my $par = 0;
-            for (my $i = 0; $i < length($left); ++$i) {
-                my $c = substr($left, $i, 1);
-                if ($c eq '(') {
-                    $par++;
-                } elsif ($c eq ')') {
-                    $par--;
-                    if ($par == 0) {
-                        $thr++;
-                    }
-                }
-            }
-            $parsed = $parsed->[$thr];
-        }
-        return $parsed;
-    }
-    return $ok;
-}
-
-sub _store_helper {
-    my ($self, $msg, $flags, $cmd) = @_;
-    if (ref $msg eq 'ARRAY') {
-        $msg = join(',', @$msg);
-    }
-    unless (ref $flags) {
-        $flags = [ $flags ];
-    }
-    $flags = '(' . join(' ', @$flags) . ')';
-    $self->_tell_imap(STORE => "$msg $cmd $flags", 1);
-}
-
-sub store {
-    my ($self, $msg, $flags) = @_;
-    $self->_store_helper($msg, $flags, 'FLAGS');
-}
-
-sub add_flags {
-    my ($self, $msg, $flags) = @_;
-    $self->_store_helper($msg, $flags, '+FLAGS');
-}
-
-sub del_flags {
-    my ($self, $msg, $flags) = @_;
-    $self->_store_helper($msg, $flags, '-FLAGS');
-}
-
-sub delete_message {
-    my ($self, $msg) = @_;
-    $self->add_flags($msg, '\\Deleted');
-}
-
-sub expunge {
-    my ($self) = @_;
-    my ($ok, $lines) = $self->_tell_imap('EXPUNGE' => undef, 1);
-    if ($ok && $lines && @$lines) {
-        my $ret = $lines->[0][0];
-        if ($ret =~ /^\*\s+(\d+)\s+EXPUNGE/) {
-            return $1 + 0;
-        }
-    }
-    return $ok ? -1 : undef;
-}
-
 sub last_error {
     my ($self) = @_;
     return $self->{_error};
@@ -579,15 +362,12 @@ sub _get_server {
 
 sub _get_socket {
     my ($self) = @_;
-    my $socket = $self->{socket} ||= ($self->{ssl} ? 'IO::Socket::SSL' : 'IO::Socket::INET')->new(
+    return $self->{socket} ||= ($self->{ssl} ? 'IO::Socket::SSL' : 'IO::Socket::INET')->new(
         PeerAddr => $self->_get_server,
         PeerPort => $self->_get_port,
         Timeout  => $self->_get_timeout,
         Proto    => 'tcp',
-        Blocking => 1,
     );
-    $socket->sockopt(SO_KEEPALIVE, 1);
-    return $socket;
 }
 
 sub _get_next_id {
@@ -599,56 +379,16 @@ sub _socket_getline {
     return $_[0]->_get_socket->getline;
 }
 
-sub _socket_write {
-    my $self = shift;
-    $self->_get_socket->write(@_);
-}
-
 sub _send_cmd {
     my ($self, $cmd, $args) = @_;
-
     local $\;
-    use bytes;
     my $id   = $self->_get_next_id;
     if ($self->uid_mode && exists($UID_COMMANDS{$cmd})) {
         $cmd = "UID $cmd";
     }
-    my @literals = ();
-    if (ref $args eq 'ARRAY') {
-        # may contain literals
-        foreach (@$args) {
-            if (ref $_ eq 'SCALAR') {
-                push @literals, $_;
-                $_ = '{' . length($$_) . "}\r\n";
-            }
-        }
-        $args = join('', @$args);
-    }
-    my $socket = $self->_get_socket;
-    if (@literals == 0) {
-        $cmd = "NIC$id $cmd" . ($args ? " $args" : '') . "\r\n";
-        $self->_socket_write($cmd);
-    } else {
-        $cmd = "NIC$id $cmd ";
-        $self->_socket_write($cmd);
-        my @split = split(/\r\n/, $args);
-
-        my $ea = each_array(@split, @literals);
-        while (my ($tmp, $lit) = $ea->()) {
-            $self->_socket_write($tmp . "\r\n");
-            my $line = $self->_socket_getline;
-            # print STDERR "$line - $tmp\n";
-            if ($line =~ /^\+/) {
-                $self->_socket_write($$lit);
-            } else {
-                $self->{_error} = "Expected continuation, got: $line";
-                # XXX: it's really bad if we get here, what to do?
-                return undef;
-            }
-        }
-        $self->_socket_write("\r\n"); # end of command!
-    }
-    $socket->flush;
+    $cmd = "NIC$id $cmd" . ($args ? " $args" : '') . "\r\n";
+    # print STDERR $cmd;
+    $self->_get_socket->syswrite($cmd);
     return "NIC$id";
 }
 
@@ -658,10 +398,8 @@ sub _read_literal {
     my $buf;
     my @lines = ();
     my $sock = $self->_get_socket;
-    # print STDERR "\033[1;31m ::: Reading $count bytes ::: \033[0m \n";
     while ($count > 0) {
-        my $read = $sock->read($buf, min($count, $READ_BUFFER));
-        # print STDERR "GOT $read / $buf";
+        my $read = $sock->sysread($buf, min($count, $SYSREAD_BUFFER));
         $count -= $read;
         last if !$read;
         push @lines, $buf;
@@ -694,43 +432,21 @@ sub _cmd_ok2 {
     } elsif ($res =~ /^(NIC\d+)\s+(?:NO|BAD)(?:\s+(.+))?/i) {
         my $id = $1;
         my $error = $2 || 'unknown error';
-        $self->{_error} = $error;
         return ($id, 0, $error);
     }
     return ();
 }
 
-sub _reconnect_if_needed {
-    my ($self, $force) = @_;
-    if ($force || !$self->_get_socket->connected) {
-        $self->{socket} = undef;
-        $self->{greeting} = $self->_socket_getline;
-        if ($self->login) {
-            if ($self->{selected_folder}) {
-                $self->select($self->{selected_folder});
-            }
-            return 1;
-        }
-        return undef;
-    }
-    return 0;
-}
-
 sub _tell_imap {
     my ($self, $cmd, $args, $do_notf) = @_;
 
     $cmd = uc $cmd;
+    $self->_send_cmd($cmd, $args);
 
-    my ($lineparts, $ok, $res);
-
-  RETRY1: {
-        $self->_send_cmd($cmd, $args);
-        redo RETRY1 if $self->_reconnect_if_needed;
-    }
-
-    $lineparts = [];
-    while ($res = $self->_socket_getline) {
-        # print STDERR ">>>>$res<<<<<\n";
+    my $lineparts = [];
+    my $ok;
+    while (my $res = $self->_socket_getline) {
+        # print STDERR "$res";
         if ($res =~ /^\*/) {
             push @$lineparts, []; # this is a new line interesting in itself
         }
@@ -748,9 +464,6 @@ sub _tell_imap {
             }
         }
     }
-    unless (defined $res) {
-        goto RETRY1 if $self->_reconnect_if_needed(1);
-    }
 
     if ($do_notf) {
         no warnings 'uninitialized';
@@ -768,7 +481,7 @@ sub _tell_imap {
             # 3. Other notifications will have a first token different
             #    from the running command
 
-            if ( $text =~ /^\*\s+\d+\s+FETCH\s*\(\s*FLAGS\s*\([^\)]*?\)\)/
+            if ( $text =~ /^\*\s+\d+\s+FETCH\s*\(\s*FLAGS\s*\(.*?\)\)/
                    || $text !~ /^\*\s+(?:\d+\s+)?$cmd/ ) {
                 my $tokens = _parse_tokens($line);
                 if ($self->_handle_notification($tokens, 1)) {
@@ -791,22 +504,15 @@ sub _tell_imap {
 sub _tell_imap2 {
     my ($self, @cmd) = @_;
 
-    my %results;
-    my @ids;
-
-  RETRY2: {
-        @ids = ();
-        foreach (@cmd) {
-            push @ids, $self->_send_cmd($_);
-            redo RETRY2 if $self->_reconnect_if_needed;
-        }
+    my @ids = ();
+    foreach (@cmd) {
+        push @ids, $self->_send_cmd($_);
     }
 
-    %results = ();
+    my %results;
     for (0..$#cmd) {
         my $lineparts = [];
-        my $res;
-        while ($res = $self->_socket_getline) {
+        while (my $res = $self->_socket_getline) {
             # print STDERR "2: $res";
             if ($res =~ /^\*/) {
                 push @$lineparts, []; # this is a new line interesting in itself
@@ -826,9 +532,6 @@ sub _tell_imap2 {
                 }
             }
         }
-        unless (defined $res) {
-            goto RETRY2 if $self->_reconnect_if_needed(1);
-        }
     }
 
     my @ret = @results{@ids};
@@ -871,14 +574,14 @@ sub _string_unquote {
 # note that lists are represented as arrays.
 #
 sub _parse_tokens {
-    my ($input, $no_deref) = @_;
+    my ($input) = @_;
 
     my @tokens = ();
     my @stack = (\@tokens);
 
     while (my $text = shift @$input) {
         if (ref $text) {
-            push @{$stack[-1]}, ($no_deref ? $text : $$text);
+            push @{$stack[-1]}, $$text;
             next;
         }
         while (1) {
@@ -887,8 +590,6 @@ sub _parse_tokens {
                 my $sub = [];
                 push @{$stack[-1]}, $sub;
                 push @stack, $sub;
-            } elsif ($text =~ m/\G(BODY\[[a-zA-Z0-9._() -]*\])/gc) {
-                push @{$stack[-1]}, $1; # let's consider this an atom too
             } elsif ($text =~ m/\G[])]/gc) {
                 pop @stack;
             } elsif ($text =~ m/\G\"((?:\\.|[^\"\\])*)\"/gc) {
@@ -914,6 +615,27 @@ sub _parse_tokens {
     return \@tokens;
 }
 
+sub _make_summary {
+    my ($self, $tokens) = @_;
+    ## in form: [ '*', ID, 'FETCH', [ tokens ]]
+
+    if ($tokens->[2] eq 'FETCH') {
+        my %hash = @{$tokens->[3]};
+        if ($hash{ENVELOPE}) {
+            # full fetch
+            return Net::IMAP::Client::MsgSummary->new(\%hash);
+        } else {
+            # 'FETCH' (probably FLAGS) notification!
+            $self->_handle_notification($tokens);
+            return undef;
+        }
+    } else {
+        # notification!
+        $self->_handle_notification($tokens);
+        return undef;
+    }
+}
+
 sub _handle_notification {
     my ($self, $tokens, $reverse) = @_;
 
@@ -932,9 +654,6 @@ sub _handle_notification {
                     --$sf->{messages};
                     $not->{deleted} = 1;
                 }
-                if ($data{UID}) {
-                    $not->{uid} = $data{UID};
-                }
             }
 
         } elsif ($tokens->[2] eq 'EXISTS') {
@@ -988,14 +707,12 @@ Net::IMAP::Client - Not so simple IMAP client library
     use Net::IMAP::Client;
 
     my $imap = Net::IMAP::Client->new(
-
         server => 'mail.you.com',
         user   => 'USERID',
         pass   => 'PASSWORD',
         ssl    => 1,          # (use SSL? default no)
         port   => 993         # (but defaults are sane)
-
-    ) or die "Could not connect to IMAP server";
+    );
 
     # everything's useless if you can't login
     $imap->login or
@@ -1051,22 +768,6 @@ Net::IMAP::Client - Not so simple IMAP client library
     # fetch single attachment (message part)
     my $data = $imap->get_part_body($msg_id, '1.2');
 
-    # fetch multiple attachments at once
-    my $hash = $imap->get_parts_bodies($msg_id, [ '1.2', '1.3', '2.2' ]);
-    my $part1_2 = $hash->{'1.2'};
-    my $part1_3 = $hash->{'1.3'};
-    my $part2_2 = $hash->{'2.2'};
-    print $$part1_2;              # need to dereference it
-
-    # copy messages between folders
-    $imap->select('INBOX');
-    $imap->copy(\@msg_ids, 'Archive');
-
-    # delete messages ("Move to Trash")
-    $imap->copy(\@msg_ids, 'Trash');
-    $imap->add_flags(\@msg_ids, '\\Deleted');
-    $imap->expunge;
-
 =head1 DESCRIPTION
 
 Net::IMAP::Client provides methods to access an IMAP server.  It aims
@@ -1080,9 +781,6 @@ as a fresh module.  Still, the design is influenced by
 Net::IMAP::Simple and I even stole a few lines of code from it ;-)
 (very few, honestly).
 
-This software was developed for creating a web-based email (IMAP)
-client: www.xuheki.com.  Xhueki uses Net::IMAP::Client.
-
 =head1 API REFERENCE
 
 Unless otherwise specified, if a method fails it returns I<undef> and
@@ -1090,7 +788,7 @@ you can inspect the error by calling $imap->last_error.  For a
 successful call most methods will return a meaningful value but
 definitely not I<undef>.
 
-=head2 new(%args)  # constructor
+=head2 new( %args )  # constructor
 
     my $imap = Net::IMAP::Client->new(%args);
 
@@ -1116,7 +814,7 @@ Pass a true value if you want to use IO::Socket::SSL
 
 =item - B<uid_mode> (BOOL, optional, default TRUE)
 
-Wether to use UID command (see RFC3501).  Recommended.
+Wether to use UID command (see RFC2060).  Recommended.
 
 =item - B<socket> (IO::Handle, optional)
 
@@ -1132,7 +830,7 @@ $imap->login for that.
 
 Returns the last error from the IMAP server.
 
-=head2 login($user, $pass)
+=head2 login( $user, $pass )
 
 Login to the IMAP server.  You can pass $user and $pass here if you
 wish; if not passed, the values used in constructor will be used.
@@ -1147,13 +845,13 @@ for C<logout>.
 =head2 noop
 
 "Do nothing" method that calls the IMAP "NOOP" command.  It returns a
-true value upon success, I<undef> otherwise.
+true value upon success, L<undef> otherwise.
 
 This method fetches any notifications that the server might have for
 us and you can get them by calling $imap->notifications.  See the
-L</notifications()> method.
+L<notifications()> method.
 
-=head2 capability() / capability(qr/^SOMETHING/)
+=head2 capability(), capability( qr/^SOMETHING/ )
 
 With no arguments, returns an array of all capabilities advertised by
 the server.  If you're interested in a certain capability you can pass
@@ -1165,7 +863,7 @@ a RegExp.  E.g. to check if this server knows 'SORT', you can do this:
 
 This data is cached, the server will be only hit once.
 
-=head2 select($folder)
+=head2 select( $folder )
 
 Selects the current IMAP folder.  On success this method also records
 some information about the selected folder in a hash stored in
@@ -1189,13 +887,13 @@ Total number of recent messages in this folder
 =item - B<sflags>
 
 Various other flags here, such as PERMANENTFLAGS of UIDVALIDITY.  You
-might want to take a look at RFC3501 at this point. :-p
+might want to take a look at RFC2060 at this point. :-p
 
 =back
 
 This method is basically stolen from Net::IMAP::Simple.
 
-=head2 status($folder), status(\@folders)
+=head2 status( $folder ), status( \@folders )
 
 Returns the status of the given folder(s).
 
@@ -1255,9 +953,9 @@ It maps folder name to an hash ref containing the following:
   - sep   -- one character containing folder hierarchy separator
   - name  -- folder name (same as the key -- thus redundant)
 
-=head2 seq_to_uid(@sequence_ids)
+=head2 seq_to_uid( @sequence_ids )
 
-I recommend usage of UID-s only (see L</uid_mode>) but this isn't
+I recomment usage of UID-s only (see L<uid_mode>) but this isn't
 always possible.  Even when C<uid_mode> is on, the server will
 sometimes return notifications that only contain message sequence
 numbers.  To convert these to UID-s you can use this method.
@@ -1265,14 +963,14 @@ numbers.  To convert these to UID-s you can use this method.
 On success it returns an hash reference which maps sequence numbers to
 message UID-s.  Of course, on failure it returns I<undef>.
 
-=head2 search($criteria, $sort, $charset)
+=head2 search( $criteria, $sort, $charset )
 
 Executes the "SEARCH" or "SORT" IMAP commands (depending on wether
 $sort is I<undef>) and returns the results as an array reference
 containing message ID-s.
 
 Note that if you use C<$sort> and the IMAP server doesn't have this
-capability, this method will fail.  Use L</capability> to investigate.
+capability, this method will fail.  Use L<capability> to investigate.
 
 =over
 
@@ -1282,7 +980,7 @@ Can be a string, in which case it is passed literally to the IMAP
 command (which can be "SEARCH" or "SORT").
 
 It can also be an hash reference, in which case keys => values are
-collected into a string and values are properly quoted, i.e.:
+collected into a string and values are properly quotes, i.e.:
 
    { subject => 'foo',
      from    => 'bar' }
@@ -1333,7 +1031,7 @@ servers.
 
 =back
 
-=head2 get_rfc822_body($msg_id)
+=head2 get_rfc822_body( $msg_id )
 
 Fetch and return the full RFC822 body of the message.  B<$msg_id> can
 be a scalar but also an array of ID-s.  If it's an array, then all
@@ -1356,10 +1054,10 @@ Examples:
     my $more = $imap->get_rfc822_body([ 11, 12, 13 ]);
     print $$_ foreach @$more;
 
-=head2 get_part_body($msg_id, $part_id)
+=head2 get_part_body( $msg_id, $part_id )
 
 Fetches and returns the body of a certain part of the message.  Part
-ID-s look like '1' or '1.1' or '2.3.1' etc. (see RFC3501 [1], "FETCH
+ID-s look like '1' or '1.1' or '2.3.1' etc. (see RFC2060 [1], "FETCH
 Command").
 
 =head3 Scalar reference
@@ -1370,7 +1068,7 @@ some time passing around potentially large data from Perl subroutines.
 
 =head3 Undecoded
 
-One other thing to note is that the data is not decoded.  One simple
+One other thing to note is that the data is not undecoded.  One simple
 way to decode it is use Email::MIME::Encodings, i.e.:
 
     use Email::MIME::Encodings;
@@ -1384,7 +1082,7 @@ way to decode it is use Email::MIME::Encodings, i.e.:
 
 See get_summaries below.
 
-=head2 get_parts_bodies($msg_id, \@part_ids)
+=head2 get_parts_bodies( $msg_id, \@part_ids )
 
 Similar to get_part_body, but this method is capable to retrieve more
 parts at once.  It's of course faster than calling get_part_body for
@@ -1395,7 +1093,7 @@ data).  Again, the data is not unencoded.
     my $parts = $imap->get_parts_bodies(10, [ '1.1', '1.2', '2.1' ]);
     print ${$parts->{'1.1'}};
 
-=head2 get_summaries($msg) / get_summaries(\@msgs)
+=head2 get_summaries( $msg ), get_summaries( \@msgs )
 
 Fetches, parses and returns "message summaries".  $msg can be an array
 ref, or a single id.  The return value is an array reference (in
@@ -1597,38 +1295,6 @@ command and retrieve only those headers that you are interested in
 FETCH (FLAGS INTERNALDATE RFC822.SIZE ENVELOPE BODYSTRUCTURE)).  It's
 acceptably fast even for huge folders.
 
-=head2 fetch($msg_id, $attributes)
-
-This is a low level interface to FETCH.  It calls the imap FETCH
-command and returns a somewhat parsed hash of the results.
-
-C<$msg_id> can be a single message ID or an array of IDs.  If a single
-ID is given, the return value will be a hash reference containing the
-requested values.  If C<$msg_id> is an array, even if it contains a
-single it, then the return value will be an array of hashes.
-
-C<$attributes> is a string of attributes to FETCH, separated with a
-space, or an array (ref) of attributes.
-
-Examples:
-
-# retrieve the UID of the most recent message
-
-    my $last_uid = $imap->fetch('*', 'UID')->{UID};
-
-# fetch the flags of the first message
-
-    my $flags = $imap->fetch(1, 'FLAGS')->{FLAGS};
-
-# fetch flags and some headers (Subject and From)
-
-    my $headers = 'BODY[HEADER.FIELDS (Subject From)]';
-    my $results = $imap->fetch([1, 2, 3], "FLAGS $headers");
-    foreach my $hash (@$results) {
-        print join(" ", @{$hash->{FLAGS}}), "\n";
-        print $hash->{$headers}, "\n";
-    }
-
 =head2 notifications()
 
 The IMAP server may send various notifications upon execution of
@@ -1641,7 +1307,6 @@ Each element in this array (notification) is a hash reference
 containing one or more or the following:
 
   - seq       : the *sequence number* of the changed message
-  - uid       : UID of the changed message (NOT ALWAYS available!)
   - flags     : new flags for this message
   - deleted   : when the \Deleted flag was set for this message
   - messages  : new number of messages in this folder
@@ -1657,12 +1322,7 @@ the B<\Deleted> flag is present, you also get C<deleted> true.
 
 C<seq> is a message sequence number.  Pretty dumb, I think it's
 preferable to work with UID-s, but that's what the IMAP server
-reports.  In some cases the UID I<might> be readily available (i.e. my
-IMAP server sends notifications in the same body as a response to,
-say, a FETCH BODY command), but when it's not, you have to rely on
-seq_to_uid().  B<Note> that when C<destroyed> is true, the message has
-been B<expunged>; there is no way in this case to retrieve the UID so
-you have to rely solely on C<seq> in order to update your caches.
+reports.  To get UID-s call seq_to_uid.
 
 When C<flags> is present but no C<seq>, it means that the list of
 available flags for the C<folder> has changed.
@@ -1679,130 +1339,27 @@ of them will receive "RECENT" notifications; others will have to rely
 on "EXISTS" to tell when new messages have arrived.  Therefore I can
 only say that "RECENT" is useless and I advise you to ignore it.
 
-=head2 append($folder, $rfc822, $flags, $date)
-
-Appends a message to the given C<$folder>.  You must pass the full
-RFC822 body in C<$rfc822>.  C<$flags> and C<$date> are optional.  If
-you pass C<$flags>, it must be an array of strings specifying the
-initial flags of the appended message.  If I<undef>, the message will
-be appended with an empty flag set, which amongst other things means
-that it will be regarded as an C<\Unseen> message.
-
-C<$date> specifies the INTERNALDATE of the appended messge.  If
-I<undef> it will default to the current date/time.  B<NOTE:> this
-functionality is not tested; C<$date> should be in a format understood
-by IMAP.
-
-=head2 get_flags($msg_id) / get_flags(\@msg_ids)
-
-Returns the flags of one or more messages.  The return value is an
-array (reference) if one message ID was passed, or a hash reference if
-an array (of one or more) message ID-s was passed.
-
-When an array was passed, the returned hash will map each message UID
-to an array of flags.
-
-=head2 store($msg, $flag) / store(\@msgs, \@flags)
-
-Resets FLAGS of the given message(s) to the given flag(s).  C<$msg>
-can be an array of ID-s (or UID-s), or a single (U)ID.  C<$flags> can
-be a single string, or an array reference as well.
-
-Note that the folder where these messages reside must have been
-already selected.
-
-Examples:
-
-    $imap->store(10, '\\Seen');
-    $imap->store([11, 12], '\\Deleted');
-    $imap->store(13, [ '\\Seen', '\\Answered' ]);
-
-The IMAP specification defines certain reserved flags (they all start
-with a backslash).  For example, a message with the flag C<\Deleted>
-should be regarded as deleted and will be permanently discarded by an
-EXPUNGE command.  Although, it is possible to "undelete" a message by
-removing this flag.
-
-The following reserved flags are defined by the IMAP spec:
-
-    \Seen
-    \Answered
-    \Flagged
-    \Deleted
-    \Draft
-    \Recent
-
-The C<\Recent> flag is considered "read-only" -- you cannot add or
-remove it manually; the server itself will do this as appropriate.
-
-=head2 add_flags($msg, $flag) / add_flags(\@msgs, \@flags)
-
-Like store() but it doesn't reset all flags -- it just specifies which
-flags to B<add> to the message.
-
-=head2 del_flags($msg, $flag) / del_flags(\@msgs, \@flags)
-
-Like store() / add_flags() but it B<removes> flags.
-
-=head2 delete_message($msg) / delete_message(\@msgs)
-
-Stores the \Deleted flag on the given message(s).  Equivalent to:
-
-    $imap->add_flags(\@msgs, '\\Deleted');
-
-=head2 expunge()
-
-Permanently removes messages that have the C<\Deleted> flag set from
-the current folder.
-
-=head2 copy($msg, $folder) / copy(\@msg_ids, $folder)
-
-Copies message(s) from the selected folder to the given C<$folder>.
-You can pass a single message ID, or an array of message ID-s.
-
-=head2 create_folder($folder)
-
-Creates the folder with the given name.
-
-=head2 delete_folder($folder)
-
-Deletes the folder with the given name.  This works a bit different
-from the IMAP specs.  The IMAP specs says that any subfolders should
-remain intact.  This method actually deletes subfolders recursively.
-Most of the time, this is What You Want.
-
-Note that all messages in C<$folder>, as well as in any subfolders,
-are permanently lost.
-
-=head2 get_threads($algorithm, $msg_id)
-
-Returns a "threaded view" of the current folder.  Both arguments are
-optional.
-
-C<$algorithm> should be I<undef>, "REFERENCES" or "SUBJECT".  If
-undefined, "REFERENCES" is assumed.  This selects the threading
-algorithm, as per IMAP THREAD AND SORT extensions specification.  I
-only tested "REFERENCES".
-
-C<$msg_id> can be undefined, or a message ID.  If it's undefined, then
-a threaded view of the whole folder will be returned.  If you pass a
-message ID, then this method will return the top-level thread that
-contains the message.
-
-The return value is an array which actually represents threads.
-Elements of this array are message ID-s, or other arrays (which in
-turn contain message ID-s or other arrays, etc.).  The first element
-in an array will represent the start of the thread.  Subsequent
-elements are child messages or subthreads.
-
-An example should help (FIXME).
-
 =head1 TODO
 
+There's a bunch of missing functionality which should be quite easy to
+add but I didn't get to it yet.  Such as create/delete folders.  If
+you need it urgently, feel free to add it and send me a patch,
+otherwise wait until I need it. :-)
+
 =over
 
 =item - authentication schemes other than plain text (B<help wanted>)
 
+=item - append/delete messages
+
+=item - create/remove mailboxes
+
+=item - expunge folder
+
+=item - support THREAD operation
+
+=item - reconnect/relogin when connection lost
+
 =item - better error handling?
 
 =back
@@ -1813,19 +1370,18 @@ L<Net::IMAP::Simple>, L<Mail::IMAPClient>, L<Mail::IMAPTalk>
 
 L<Email::Simple>, L<Email::MIME>
 
-RFC3501 [1] is a must read if you want to do anything fancier than
+RFC2060 [1] is a must read if you want to do anything fancier than
 what this module already supports.
 
 =head1 REFERENCES
 
-[1] http://ietfreport.isoc.org/rfc/rfc3501.txt
+[1] http://ietfreport.isoc.org/rfc/rfc2060.txt
 
 [2] http://ietfreport.isoc.org/all-ids/draft-ietf-imapext-sort-20.txt
 
 =head1 AUTHOR
 
 Mihai Bazon, <mihai.bazon@gmail.com>
-    http://www.xuheki.com/
     http://www.dynarchlib.com/
     http://www.bazon.net/mishoo/
 
